#include "ExampleDockPanel.h"
#include "ui_ExampleDockPanel.h"

#include "pqActiveObjects.h"
#include "pqView.h"
#include "vtkNew.h"
#include "vtkSMParaViewPipelineControllerWithRendering.h"
#include "vtkSMProperty.h"
#include "vtkSMPropertyHelper.h"
#include "vtkSMProxyListDomain.h"
#include "vtkSMSessionProxyManager.h"
#include "vtkSMSourceProxy.h"
#include "vtkSMViewProxy.h"

namespace
{
  vtkSMProxy* getSphereSource(vtkSMProxy* glyph)
    {
    vtkSMProxyListDomain* pld = vtkSMProxyListDomain::SafeDownCast(
      glyph->GetProperty("Source")->FindDomain("vtkSMProxyListDomain"));
    Q_ASSERT(pld);

    for (unsigned int cc=0, max=pld->GetNumberOfProxies(); cc < max; cc++)
      {
      if (pld->GetProxyName(cc) && strcmp(pld->GetProxyName(cc), "SphereSource") == 0)
        {
        return pld->GetProxy(cc);
        }
      }
    return NULL;
    }
}


void ExampleDockPanel::constructor()
{
  this->setWindowTitle("Example Dock Panel");
  QWidget* t_widget = new QWidget(this);
  Ui::ExampleDockPanel ui;
  ui.setupUi(t_widget);
  this->setWidget(t_widget);

  this->connect(ui.pushButton, SIGNAL(clicked()), SLOT(createGlyphedCylinder()));
}

//-----------------------------------------------------------------------------
void ExampleDockPanel::createGlyphedCylinder()
{
  vtkSMSessionProxyManager* pxm = pqActiveObjects::instance().proxyManager();
  if (!pxm)
    {
    qCritical("No active session.");
    return;
    }

  vtkNew<vtkSMParaViewPipelineControllerWithRendering> controller;

  // Create and initialize the cylinder source.
  vtkSMSourceProxy* cylinder = vtkSMSourceProxy::SafeDownCast(
    pxm->NewProxy("sources", "CylinderSource"));
  controller->InitializeProxy(cylinder);
  vtkSMPropertyHelper(cylinder, "Resolution").Set(18);
  cylinder->UpdateVTKObjects();
  controller->RegisterPipelineProxy(cylinder);
  cylinder->Delete();

  // update pipeline, so that glyph will be initialized with good defaults.
  cylinder->UpdatePipeline();

  // Create and initialize the glyph filter
  vtkSMSourceProxy* glyph = vtkSMSourceProxy::SafeDownCast(
    pxm->NewProxy("filters", "Glyph"));
  controller->PreInitializeProxy(glyph);
  vtkSMPropertyHelper(glyph, "Input").Set(cylinder);
  vtkSMPropertyHelper(glyph, "Source").Set(getSphereSource(glyph));
  cylinder->UpdateVTKObjects();
  controller->PostInitializeProxy(glyph);
  controller->RegisterPipelineProxy(glyph);
  glyph->Delete();

  controller->Show(glyph, 0,
    pqActiveObjects::instance().activeView()->getViewProxy());
  pqActiveObjects::instance().activeView()->render();
}
